''' <summary>
''' HSL-Farbe
''' </summary>
''' <remarks>Farbraum des Farbmodells, bei dem man die Farbe mit Hilfe des Farbtons (englisch hue), der Sttigung (saturation) und der relativen Helligkeit (lightness) definiert</remarks>
Public Class HSLColor

#Region " Private Felder "
    Private m_Hue As Double
    Private m_Saturation As Double
    Private m_Lightness As Double
#End Region

#Region " Konstruktoren "
    ''' <summary>
    ''' Konstruktor fr Initialisierung mit fest stehenden HSL-Werten
    ''' </summary>
    ''' <param name="Hue">Farbton - Wert zwischen 0.0 und 1.0</param>
    ''' <param name="Saturation">Sttigung - Wert zwischen 0.0 und 1.0</param>
    ''' <param name="Lightness">relative Helligkeit - Wert zwischen 0.0 und 1.0</param>
    Public Sub New(ByVal Hue As Double, ByVal Saturation As Double, ByVal Lightness As Double)
        m_Hue = Hue
        m_Saturation = Saturation
        m_Lightness = Lightness
    End Sub

    ''' <summary>
    ''' Konstruktor fr Initialisierung mit einer bekannten HSL-Farbe
    ''' </summary>
    ''' <param name="Original">bekannte HSL-Farbe</param>
    Public Sub New(ByVal Original As HSLColor)
        m_Hue = Original.Hue
        m_Saturation = Original.Saturation
        m_Lightness = Original.Lightness
    End Sub
    ''' <summary>
    ''' Konstruktor fr Initialisierung mit einer Color-Struktur
    ''' </summary>
    ''' <param name="Col">Color-Struktur</param>
    Public Sub New(ByVal Col As Color)
        Dim Max, Min, Diff, Sum As Integer

        If Col.R > Col.G Then
            Max = Col.R
            Min = Col.G
        Else
            Max = Col.G
            Min = Col.R
        End If

        If Col.B > Max Then
            Max = Col.B
        ElseIf Col.B < Min Then
            Min = Col.B
        End If

        Diff = Max - Min
        Sum = Max + Min

        Me.Lightness = CDbl(Max) / 255.0R

        If Max <> 0 Then
            Me.Saturation = CDbl(Diff) / CDbl(Max)
        End If

        Dim q As Double
        If Diff <> 0 Then
            q = CDbl(60) / CDbl(Diff)
        End If

        If Max = Col.R Then
            If Col.G < Col.B Then
                Me.Hue = (360.0R + q * (CDbl(Col.G) - CDbl(Col.B))) / 360.0R
            Else
                Me.Hue = (q * (CDbl(Col.G) - CDbl(Col.B))) / 360.0R
            End If
        ElseIf Max = Col.G Then
            Me.Hue = (120.0R + q * (CDbl(Col.B) - CDbl(Col.R))) / 360.0R
        ElseIf Max = Col.B Then
            Me.Hue = (240.0R + q * (CDbl(Col.R) - CDbl(Col.G))) / 360.0R
        End If
    End Sub
#End Region

#Region " Properties "
    ''' <summary>
    ''' Farbton
    ''' </summary>
    ''' <remarks>Wert zwischen 0.0 und 1.0</remarks>
    Public Property Hue() As Double
        Get
            Return m_Hue
        End Get
        Set(ByVal value As Double)
            m_Hue = MinMax(value)
        End Set
    End Property

    ''' <summary>
    ''' Sttigung
    ''' </summary>
    ''' <remarks>Wert zwischen 0.0 und 1.0</remarks>
    Public Property Saturation() As Double
        Get
            Return m_Saturation
        End Get
        Set(ByVal value As Double)
            m_Saturation = MinMax(value)
        End Set
    End Property

    ''' <summary>
    ''' relative Helligkeit
    ''' </summary>
    ''' <remarks>Wert zwischen 0.0 und 1.0</remarks>
    Public Property Lightness() As Double
        Get
            Return m_Lightness
        End Get
        Set(ByVal value As Double)
            m_Lightness = MinMax(value)
        End Set
    End Property
#End Region

#Region " Methoden "
    ''' <summary>
    ''' Interne Methode, die absichert, dass H, S und L nur einen Wert zwischen 0.0R und 1.0R annehmen knnen.
    ''' </summary>
    Private Function MinMax(ByVal Val As Double) As Double
        Return Math.Max(Math.Min(Val, 1.0R), 0.0R)
    End Function
    ''' <summary>
    '''  Interne Methode die fr die ToRGB-Methode bentigt wird. Liefert einen Integer-Wert zwischen 0 und 255.
    ''' </summary>
    ''' <param name="Val">beliebiger Double-Wert</param>
    ''' <returns>Integer-Wert zwischen 0 und 255</returns>
    Private Function RoundColValue(ByVal Val As Double) As Integer
        Return CInt(Math.Round(Math.Max(Math.Min(Val, 255.0R), 0.0R)))
    End Function
    ''' <summary>
    ''' Gibt eine Color-Struktur zurck
    ''' </summary>
    ''' <returns>Color-Struktur</returns>
    Public Function ToRGB() As Color
        Dim Max, Mid, Min, q As Double

        Max = RoundColValue(m_Lightness * 255.0R)
        Min = RoundColValue((1.0R - m_Saturation) * (m_Lightness / 1.0R) * 255.0R)
        q = (Max - Min) / 255.0R

        Dim R, G, B As Integer

        If m_Hue >= 0.0R AndAlso m_Hue <= 1.0R / 6.0R Then
            Mid = RoundColValue(m_Hue * q * 1530.0R + Min)
            R = CInt(Max)
            G = CInt(Mid)
            B = CInt(Min)

        ElseIf m_Hue <= 1.0R / 3.0R Then
            Mid = RoundColValue(-((m_Hue - 1.0R / 6.0R) * q) * 1530.0R + Max)
            R = CInt(Mid)
            G = CInt(Max)
            B = CInt(Min)

        ElseIf m_Hue <= 0.5R Then
            Mid = RoundColValue(((m_Hue - 1.0R / 3.0R) * q) * 1530.0R + Min)
            R = CInt(Min)
            G = CInt(Max)
            B = CInt(Mid)

        ElseIf m_Hue <= 2.0R / 3.0R Then
            Mid = RoundColValue(-((m_Hue - 0.5R) * q) * 1530.0R + Max)
            R = CInt(Min)
            G = CInt(Mid)
            B = CInt(Max)

        ElseIf m_Hue <= 5.0R / 6.0R Then
            Mid = RoundColValue(((m_Hue - 2.0R / 3.0R) * q) * 1530.0R + Min)
            R = CInt(Mid)
            G = CInt(Min)
            B = CInt(Max)

        ElseIf m_Hue <= 1.0R Then
            Mid = RoundColValue(-((m_Hue - 5.0R / 6.0R) * q) * 1530.0R + Max)
            R = CInt(Max)
            G = CInt(Min)
            B = CInt(Mid)

        End If

        Return Color.FromArgb(R, G, B)
    End Function
#End Region

End Class
