Imports System.ComponentModel
''' <summary>
''' Ein Control, mit dem man die Farbe eines Pixels auf dem Bildschirm auswhlen kann
''' </summary>
Public Class EyeDropperButton
    Inherits Button
    Implements IColorControl

#Region " Private Felder / Events"
    Private m_ColorBrush As SolidBrush

    'Event, das geliefert wird, wenn die ausgewhlte Farbe von "intern" gendert wurde
    Public Event ColorChanged(ByVal Sender As IColorControl) Implements IColorControl.ColorChanged
#End Region

#Region " Konstruktoren "

    Public Sub New()
        ' Wesentliche Eigenschaften auf Start-Werte festlegen

        m_ColorBrush = New SolidBrush(Color.FromArgb(255, 0, 0))
        Me.MinimumSize = New Size(24, 24)
    End Sub

#End Region

#Region " Properties "

    ''' <summary>
    ''' Color-Struktur, die angezeigt wird oder ausgewhlt ist
    ''' </summary>
    <Description("Die ausgewhlte Farbe"), _
    Category("ColorControl")> _
    Public Property Color() As System.Drawing.Color Implements IColorControl.Color
        Get
            Return m_ColorBrush.Color
        End Get
        Set(ByVal value As System.Drawing.Color)
            If value <> Nothing Then
                Dim Col As Color = m_ColorBrush.Color
                If Col.R <> value.R OrElse Col.G <> value.G OrElse Col.B <> value.B Then
                    m_ColorBrush.Color = Color.FromArgb(255, value)
                    ' Neu Zeichnen erzwingen
                    Invalidate()
                End If
            End If
        End Set
    End Property

#End Region

#Region " Methoden "
    ''' <summary>
    ''' berschreiben der OnResize-Methode, um bei Grennderung den angezeigten Inhalt zu aktualisieren
    ''' </summary>
    Protected Overrides Sub OnResize(ByVal e As System.EventArgs)
        MyBase.OnResize(e)
        ' Control soll quadratisch bleiben
        Me.Width = Me.Height
    End Sub

    ''' <summary>
    ''' berschreiben der OnPaint-Methode, um die Inhalte dieses Controls zu zeichnen
    ''' </summary>
    ''' <param name="pevent">PaintEventArgs</param>
    Protected Overrides Sub OnPaint(ByVal pevent As System.Windows.Forms.PaintEventArgs)
        MyBase.OnPaint(pevent)

        ' Es wird auf dem Graphics-Objekt der PaintEventArgs gezeichnet
        Dim g As Graphics = pevent.Graphics

        Dim r As Rectangle = ClientRectangle
        Dim border As Integer = 5

        ' das Rechteck, in dem die farbige Box auf den Button gezeichnet werden soll
        Dim Rect As New Rectangle(r.Left + border, r.Top + border, r.Width - border * 2, r.Height - border * 2)

        ' Flche mit weiss fllen
        g.FillRectangle(Brushes.White, Rect)

        ' Rechteck um 1 verkleinern
        Rect.Inflate(-1, -1)

        ' Flche mit aktueller Farbe fllen
        g.FillRectangle(m_ColorBrush, Rect)

        ' Position fr Pipetten-Symbol ermitteln 
        Dim half As Integer = Me.Width \ 2 - 8
        ' Pipetten-Symbol zeichnen
        g.DrawImage(My.Resources.EyeDropper, New Rectangle(half, half, 16, 16))

    End Sub

    ''' <summary>
    ''' Interne Methode, die nach Click auf den Button 
    ''' die Farbauswahl startet und das Changed-Event liefert
    ''' </summary>
    Private Sub EyeDropperButton_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Click
        ' aktuelle Farbe festhalten
        Dim StartColor As Color = m_ColorBrush.Color

        ' Neue EyeDropperForm-Instanz erstellen
        Dim frm As New EyeDropperForm(StartColor)
        ' Eventhandler fr ColorChanged-Event der EyeDropperForm festlegen
        AddHandler frm.ColorChanged, AddressOf DropForm_ColorChanged
        ' EyeDropperForm anzeigen
        If frm.ShowDialog = Windows.Forms.DialogResult.OK Then
            ' Wenn EyeDropperForm erfolgreich geschlossen wurde 
            ' (durch linken Mouse-Klick oder Enter-Button)
            ' Farbe bernehmen
            m_ColorBrush.Color = frm.Color
            ' nderungs-Event liefern
            RaiseEvent ColorChanged(Me)
        Else
            ' EyeDropperForm wurde nicht mit DialogResult.OK geschlossen
            ' darum aktuellen Farb-Wert nur zurcksetzen
            m_ColorBrush.Color = StartColor
        End If
        ' Eventhandler fr ColorChanged-Event der EyeDropperForm entfernen
        RemoveHandler frm.ColorChanged, AddressOf DropForm_ColorChanged

        ' Neu Zeichnen erzwingen
        Invalidate()
    End Sub

    ''' <summary>
    ''' Reagieren auf das ColorChanged-Event der EyeDropper-Form, 
    ''' um temporr die ausgewhlte Farbe im Button anzuzeigen
    ''' </summary>
    Private Sub DropForm_ColorChanged(ByVal Sender As IColorControl)
        Dim Col As Color = Sender.Color
        Dim MyColor As Color = m_ColorBrush.Color
        If MyColor.R <> Col.R OrElse MyColor.G <> Col.G OrElse MyColor.B <> Col.B Then
            m_ColorBrush.Color = Col
            ' Neu Zeichnen erzwingen
            Invalidate()
        End If
    End Sub
#End Region

#Region " Interne Klassen "

    ''' <summary>
    ''' Hilfsklasse zum Eyedropper-Button - eine transparente Windows Form, 
    ''' mit deren Hilfe ein Farbwert eines Pixels an der aktuellen Maus-Position 
    ''' auf dem Bildschirm bestimmt werden kann
    ''' </summary>
    Private Class EyeDropperForm
        Inherits Windows.Forms.Form
        Implements IColorControl

#Region " Private Felder / Events"

        ' die aktuelle Farbe
        Private m_Color As Color

        ' Distanz zwischen MousePosition und linker oberer Ecke der Form
        Private m_Distance As Size

        ' Event, das geliefert wird, wenn die ausgewhlte Farbe von "intern" gendert wurde
        Public Event ColorChanged(ByVal Sender As IColorControl) Implements IColorControl.ColorChanged

#End Region

#Region " Properties "

        ''' <summary>
        ''' Color-Struktur, die angezeigt wird oder ausgewhlt ist
        ''' </summary>
        <Description("Die ausgewhlte Farbe"), _
        Category("ColorControl")> _
        Public Property Color() As System.Drawing.Color Implements IColorControl.Color
            Get
                Return m_Color
            End Get
            Set(ByVal value As System.Drawing.Color)
                m_Color = value
            End Set
        End Property

#End Region

#Region " Methoden "

        ''' <summary>
        ''' diese Methode legt die aktuelle Farbe neu fest.
        ''' Nur bei nderung wird ein ColorChanged-Event geliefert
        ''' </summary>
        ''' <param name="NewColor">Neue Farbe</param>
        Private Sub SetColorIntern(ByVal NewColor As Color)
            If m_Color.R <> NewColor.R OrElse m_Color.G <> NewColor.G OrElse m_Color.B <> NewColor.B Then
                m_Color = Color.FromArgb(255, NewColor)
                RaiseEvent ColorChanged(Me)
            End If
        End Sub

        ''' <summary>
        ''' bei MouseDown wird die Form geschlossen
        ''' </summary>
        Private Sub EyeDropperForm_MouseDown(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles Me.MouseDown
            If e.Button = Windows.Forms.MouseButtons.Left Then
                ' nur Farbe neu festlegen, wenn der linke MouseButton gedrckt ist
                Me.Opacity = 0.0R
                m_Color = Color.FromArgb(255, GetColor)
                ' vor dem Schlieen DialogResult.OK festlegen
                Me.DialogResult = Windows.Forms.DialogResult.OK
            End If
            ' Form schlieen
            Me.Close()
        End Sub

        ''' <summary>
        ''' diese Funktion liefert den an aktueller MousePosition ermittelten Farbwert
        ''' </summary>
        Private Function GetColor() As Color
            ' Ein Bitmap (1x1 Pixel) instanziieren
            Dim BMP As New Bitmap(1, 1, Drawing.Imaging.PixelFormat.Format32bppArgb)
            Try
                ' MousePosition auf dem Bildschirm feststellen
                Dim MP As Point = Control.MousePosition
                ' Graphics-Objekt vom Bitmap holen
                Using g As Graphics = Graphics.FromImage(BMP)
                    ' In dieses Graphics-Objekt den Bildschirminhalt an der MousePosition einfgen
                    g.CopyFromScreen(MP.X, MP.Y, 0, 0, New Size(1, 1), CopyPixelOperation.SourceCopy)
                End Using
                ' die Farbe des einzigen Pixels im Bitmap zurckgeben
                Return BMP.GetPixel(0, 0)
            Finally
                'Ressourcen fr Bitmap frei geben
                BMP.Dispose()
            End Try

        End Function

        ''' <summary>
        ''' beim Verlassen der Form durch die Maus wird die Form geschlossen
        ''' </summary>
        Private Sub EyeDropperForm_MouseLeave(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.MouseLeave
            Me.Close()
        End Sub

        ''' <summary>
        ''' Bei Bewegung der Maus ber die Form wird der aktuelle Pixel-Farbwert
        ''' geliefert und die Form relativ zur Maus-Position neu ausgerichtet
        ''' </summary>
        Private Sub EyeDropperForm_MouseMove(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles Me.MouseMove
            ' MousePosition ermitteln
            Dim MP As Point = Control.MousePosition
            ' Farbe an MousePosition ermitteln und gegebenenfalls Event fr Farbnderung liefern 
            SetColorIntern(GetColor)
            ' Form wieder so ausrichten, dass sich die Maus in der Mitte beindert.
            Me.Location = New Point(MP.X - m_Distance.Width, MP.Y - m_Distance.Height)
        End Sub

        ''' <summary>
        ''' berschreiben der OnKeyDown-Methode, auf Tastendruck zu reagieren
        ''' </summary>
        Protected Overrides Sub OnKeyDown(ByVal e As System.Windows.Forms.KeyEventArgs)
            MyBase.OnKeyDown(e)
            ' Nur bei "Enter" wird die Farbe an der MousePosition neu ermittelt und 
            ' die Form mit DialogResult.OK geschlosssen
            If e.KeyCode = Keys.Enter Then
                Me.Opacity = 0.0R
                m_Color = Color.FromArgb(255, GetColor)
                Me.DialogResult = Windows.Forms.DialogResult.OK
            End If
            Me.Close()
        End Sub
#End Region

#Region " Konstruktoren "

        Public Sub New(ByVal StartColor As Color)
            ' Wesentliche Eigenschaften auf Start-Werte festlegen

            ' DialogResult auf Cancel festlegen
            Me.DialogResult = Windows.Forms.DialogResult.Cancel
            ' Pipetten-Symbol als Cursor festlegen
            Me.Cursor = New Cursor(Me.GetType(), "Eyedropper.cur")

            ' diese Form enthlt keinen Rahmen und wird in der TaskBar nicht angezeigt
            Me.FormBorderStyle = System.Windows.Forms.FormBorderStyle.None
            Me.ShowIcon = False
            Me.ShowInTaskbar = False
            Me.TopMost = True

            ' die Form erstreckt sich ber den gesamten aktuellen Screen,
            ' es wird so ausgerichtet, dass es sich mitten unter der 
            ' aktuellen Mouse-Position auf dem Bildschirm befindet.
            Dim MP As Point = Control.MousePosition
            Me.StartPosition = FormStartPosition.Manual
            Me.Size = My.Computer.Screen.WorkingArea.Size
            ' m_Distance enthlt den Abstand der linken oberen Ecke der Form zur Mouse-Position 
            ' (dieser Abstand wird abhngig von der Mouse-Postion immer wieder hergestellt)
            m_Distance = New Size(Me.Size.Width \ 2, Me.Size.Height \ 2)
            Me.Location = New Point(MP.X - m_Distance.Width, MP.Y - m_Distance.Height)

            ' Start-Farbe bernehmen
            m_Color = Color.FromArgb(StartColor.R, StartColor.G, StartColor.B)
            'Opazitt wird auf den kleinst mglichen Wert gesetzt.
            ' (0.0R scheint dazu zu fhren, dass die Form keine Events mehr liefert)
            Me.Opacity = 0.01R
        End Sub

#End Region

    End Class
#End Region

End Class
