Imports System.ComponentModel

''' <summary>
''' Ein Control, das 24 unterschiedliche Gradient-Stile zur Auswahl anbietet
''' </summary>
Public Class GradientStyleSelector
    Inherits Control

#Region " Private Felder / Events"
    ' ausgewhlter Box-Index (immer ein Wert zwischen 0 und 23)
    Private m_SelectedIndex As Integer = 0

    ' Index fr das "gehooverte" Element
    Private m_HooveredIndex As Integer = -1

    ' Gradient-Farben fr die 24 angezeigten Boxen
    Private m_Colors(23) As GradientColor

    ' Event, das bei Auswahl einer Box mit der Maus geliefert wird
    Public Event Changed()
#End Region

#Region " Konstruktoren "
    Public Sub New()
        'Initialisierung  wesentlicher Eigenschaften dieses Controls

        'Doppelpufferung aktivieren
        Me.SetStyle(ControlStyles.DoubleBuffer Or ControlStyles.UserPaint _
        Or ControlStyles.AllPaintingInWmPaint, True)

        Dim Direction As GradientDirections

        ' ein Array aus 4 x 6 Boxen mit den verfgbaren Gradienten erzeugen
        For I As Integer = 0 To 5
            Direction = CType(I, GradientDirections)
            For J As Byte = 0 To 3
                m_Colors(I * 4 + CInt(J)) = New GradientColor(Color.Red, Color.White, Direction, J)
            Next
        Next
        Me.Size = New Size(128, 188)
    End Sub
#End Region

#Region " Properties "
    ''' <summary>
    ''' erste Farbe der Gradienten im Control
    ''' </summary>
    <Description("Die erste Farbe der Gradienten im Control"), _
    Category("ColorControl")> _
    Property Color1() As Color
        Get
            Return m_Colors(0).Color1
        End Get
        Set(ByVal Value As Color)
            If m_Colors(0).Color1 <> Value Then
                ' Fr alle 24 ColorBoxen Color1 bernehmen
                For I As Integer = 0 To 23
                    m_Colors(I).Color1 = Value
                Next
                ' Neu Zeichnen erzwingen
                Invalidate()
            End If
        End Set
    End Property

    ''' <summary>
    ''' zweite Farbe der Gradienten im Control
    ''' </summary>
    <Description("Die zweite Farbe der Gradienten im Control"), _
    Category("ColorControl")> _
    Property Color2() As Color
        Get
            Return m_Colors(0).Color2
        End Get
        Set(ByVal Value As Color)
            If m_Colors(0).Color2 <> Value Then
                ' Fr alle 24 ColorBoxen Color2 bernehmen
                For I As Integer = 0 To 23
                    m_Colors(I).Color2 = Value
                Next
                ' Neu Zeichnen erzwingen
                Invalidate()
            End If
        End Set
    End Property

    ''' <summary>
    ''' Ausgewhlter Box-Index im Control
    ''' </summary>
    <Description("Ausgewhlter Box-Index im Control"), _
    Category("ColorControl")> _
    Property SelectedIndex() As Integer
        Get
            Return m_SelectedIndex
        End Get
        Set(ByVal value As Integer)
            Dim NewValue As Integer = Math.Min(Math.Max(value, 0), 23)
            If m_SelectedIndex <> NewValue Then
                m_SelectedIndex = NewValue
                m_HooveredIndex = -1
                ' Neu Zeichnen erzwingen
                Invalidate()
            End If
        End Set
    End Property

    ''' <summary>
    ''' liefert die im Control ausgewhlte GradientDirection
    ''' </summary>
    <Description("im Control ausgewhlte GradientDirection"), _
    Category("ColorControl")> _
    ReadOnly Property GradientDirection() As GradientDirections
        Get
            Return m_Colors(m_SelectedIndex).GradientDirection
        End Get
    End Property

    ''' <summary>
    ''' liefert die im Control ausgewhlte Gradient-Variante
    ''' </summary>
    <Description("im Control ausgewhlte Gradient-Variante"), _
    Category("ColorControl")> _
    ReadOnly Property Variante() As Byte
        Get
            Return m_Colors(m_SelectedIndex).Variante
        End Get
    End Property
#End Region

#Region " Methoden "
    ''' <summary>
    ''' berschreiben der OnResize-Methode, um bei Grennderung den angezeigten Inhalt zu aktualisieren
    ''' </summary>
    Protected Overrides Sub OnResize(ByVal e As System.EventArgs)
        MyBase.OnResize(e)
        Me.Size = New Size(128, 188)
    End Sub

    ''' <summary>
    ''' berschreiben der OnPaint-Methode, um die Inhalte dieses Controls zu zeichnen
    ''' </summary>
    ''' <param name="e">PaintEventArgs</param>
    Protected Overrides Sub OnPaint(ByVal e As System.Windows.Forms.PaintEventArgs)
        MyBase.OnPaint(e)

        'Anzahl Spalten
        Dim NumCols As Integer = 4

        ' Fr alle 24 Farben
        For I As Integer = 0 To 23

            ' Spalte und Zeile fr die ColorBox bestimmen
            Dim Col As Integer = I Mod NumCols
            Dim Row As Integer = I \ NumCols

            ' Zeichenrechteck fr die ColorBox ermitteln
            Dim Rect As New Rectangle(Col * 30 + 7, Row * 30 + 7, 24, 24)
            If m_SelectedIndex = I Then
                ' roten Rahmen zeichnen
                Rect.Inflate(2, 2)
                e.Graphics.FillRectangle(Brushes.Red, Rect)
                Rect.Inflate(-3, -3)
            ElseIf m_HooveredIndex = I Then
                ' Hoover-Rahmen zeichnen
                Rect.Inflate(1, 1)
                e.Graphics.FillRectangle(SystemBrushes.HotTrack, Rect)
                Rect.Inflate(-2, -2)
            Else
                ' normalen schwarzen Rahmen zeichnen
                e.Graphics.FillRectangle(Brushes.Black, Rect)
                Rect.Inflate(-1, -1)
            End If

            ' weie Flche zeichnen
            e.Graphics.FillRectangle(Brushes.White, Rect)

            ' Gradient in die ColorBox zeichnen
            Rect.Inflate(-1, -1)
            m_Colors(I).Draw(e.Graphics, Rect)
        Next

        ControlPaint.DrawVisualStyleBorder(e.Graphics, New Rectangle(0, 0, Me.Width - 1, Me.Height - 1))
    End Sub

    ''' <summary>
    ''' berschreiben der MouseDown-Methode, um ein Element auszuwhlen
    ''' </summary>
    Private Sub GradientStyleSelector_MouseDown(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles Me.MouseDown
        ' nur wenn linker Mouse-Button bettigt ist
        If e.Button = Windows.Forms.MouseButtons.Left Then
            ' Hoover-Index ermitteln
            Dim Hoover As Integer = GetHoover(e.Location)
            ' Nur wenn wirklich eine neue ColorBox ausgewhlt wurde
            If m_SelectedIndex <> Hoover AndAlso Hoover > -1 Then
                ' SelectedIndex neu festlegen
                m_SelectedIndex = Hoover
                ' Neu Zeichnen erzwingen
                Invalidate()
                ' nderungs-Event liefern
                RaiseEvent Changed()
            End If
        End If
    End Sub

    ''' <summary>
    ''' berschreiben der MouseMove-Methode, um gegebenenfalls eine GradientBox als Hoovered zu markieren
    ''' </summary>
    Private Sub GradientStyleSelector_MouseMove(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles Me.MouseMove
        ' nur wenn kein Mouse-Button bettigt ist
        If e.Button = Windows.Forms.MouseButtons.None Then
            ' Hoover-Index ermitteln
            Dim Hoover As Integer = GetHoover(e.Location)
            ' Nur wenn sich der Hoover-Index gendert hat
            If m_HooveredIndex <> Hoover Then
                ' Hoover-Index neu festlegen
                m_HooveredIndex = Hoover
                ' Neu Zeichnen erzwingen
                Invalidate()
            End If
        End If
    End Sub

    ''' <summary>
    ''' liefert den Index einer ColorBox fr eine Maus-Position
    ''' </summary>
    Private Function GetHoover(ByVal MousePosition As Point) As Integer
        If MousePosition.X < 1 OrElse MousePosition.X > Me.Width - 1 Then
            Return -1
        End If
        If MousePosition.Y < 1 OrElse MousePosition.Y > Me.Height - 1 Then
            Return -1
        End If
        For I As Integer = 0 To 3
            For J As Integer = 0 To 5
                Dim Rect As New Rectangle(I * 30 + 7, J * 30 + 7, 24, 24)
                If Rect.Contains(MousePosition) Then
                    Return J * 4 + I
                End If
            Next
        Next
        Return -1

    End Function
#End Region

End Class
